package client

import (
	"encoding/json"
	"fmt"
	. "github.com/bitly/go-simplejson"
	"gitlab.33.cn/link33/chain33-sdk-go/dapp/storage"
	"gitlab.33.cn/link33/chain33-sdk-go/types"
	"strconv"
)

// 回调解析函数
func ParseStorage(raw json.RawMessage) (interface{}, error) {
	js, err := NewJson(raw)
	if err != nil {
		return nil, err
	}
	if contentStorge, ok := js.CheckGet("contentStorage"); ok {
		contentHex, _ := contentStorge.Get("content").String()
		content, _ := types.FromHex(contentHex)
		key, _ := contentStorge.Get("key").String()
		value, _ := contentStorge.Get("value").String()
		op, _ := contentStorge.Get("op").Int()
		storage := &types.Storage{Ty: storage.TyContentStorageAction, Value: &types.Storage_ContentStorage{ContentStorage: &types.ContentOnlyNotaryStorage{
			Content:              content,
			Key:                  key,
			Op:                   int32(op),
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	if linkStorge, ok := js.CheckGet("linkStorage"); ok {
		linkHex, _ := linkStorge.Get("link").String()
		link, _ := types.FromHex(linkHex)
		key, _ := linkStorge.Get("key").String()
		value, _ := linkStorge.Get("value").String()
		storage := &types.Storage{Ty: storage.TyLinkStorageAction, Value: &types.Storage_LinkStorage{LinkStorage: &types.LinkNotaryStorage{
			Link:                 link,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	if hashStorge, ok := js.CheckGet("hashStorage"); ok {
		hashHex, _ := hashStorge.Get("hash").String()
		hash, _ := types.FromHex(hashHex)
		key, _ := hashStorge.Get("key").String()
		value, _ := hashStorge.Get("value").String()
		storage := &types.Storage{Ty: storage.TyHashStorageAction, Value: &types.Storage_HashStorage{HashStorage: &types.HashOnlyNotaryStorage{
			Hash:                 hash,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}

	if encryptStorage, ok := js.CheckGet("encryptStorage"); ok {
		contentHashHex, _ := encryptStorage.Get("contentHash").String()
		contentHash, _ := types.FromHex(contentHashHex)
		encryptContentHex, _ := encryptStorage.Get("encryptContent").String()
		encryptContent, _ := types.FromHex(encryptContentHex)
		nonceHex, _ := encryptStorage.Get("nonce").String()
		nonce, _ := types.FromHex(nonceHex)
		key, _ := encryptStorage.Get("key").String()
		value, _ := encryptStorage.Get("value").String()
		storage := &types.Storage{Ty: storage.TyEncryptStorageAction, Value: &types.Storage_EncryptStorage{EncryptStorage: &types.EncryptNotaryStorage{
			EncryptContent:       encryptContent,
			ContentHash:          contentHash,
			Nonce:                nonce,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	if encryptStorage, ok := js.CheckGet("encryptShareStorage"); ok {
		contentHashHex, _ := encryptStorage.Get("contentHash").String()
		contentHash, _ := types.FromHex(contentHashHex)
		encryptContentHex, _ := encryptStorage.Get("encryptContent").String()
		encryptContent, _ := types.FromHex(encryptContentHex)
		pubKeyHex, _ := encryptStorage.Get("pubKey").String()
		pubKey, _ := types.FromHex(pubKeyHex)
		key, _ := encryptStorage.Get("key").String()
		value, _ := encryptStorage.Get("value").String()
		storage := &types.Storage{Ty: storage.TyEncryptShareStorageAction, Value: &types.Storage_EncryptShareStorage{EncryptShareStorage: &types.EncryptShareNotaryStorage{
			EncryptContent:       encryptContent,
			ContentHash:          contentHash,
			PubKey:               pubKey,
			Key:                  key,
			Value:                value,
			XXX_NoUnkeyedLiteral: struct{}{},
			XXX_unrecognized:     nil,
			XXX_sizecache:        0,
		}}}
		return storage, nil
	}
	return nil, fmt.Errorf("unknow type!")
}

//FIXME 解析跨链事件,系统json序列化有问题，非string字段全部转化为string字段了
func ParseInterChainEvent(raw json.RawMessage)(interface{}, error){
	js, err := NewJson(raw)
	if err != nil {
		return nil, err
	}
	index,err:= strconv.ParseUint(js.Get("index").MustString(),10,64)
	if err != nil {
		return nil, err
	}
	ty,err:=strconv.ParseUint(js.Get("type").MustString(),10,64)
	if err != nil {
		return nil, err
	}
	event:=&types.InterchainEvent{
		Index:index,
		DstServiceID:js.Get("dstServiceID").MustString(),
		SrcServiceID:js.Get("srcServiceID").MustString(),
		Type:ty,
		Func:js.Get("func").MustString(),
		Args:js.Get("args").MustString(),
		Argsrb:js.Get("argsrb").MustString(),
		Argscb:js.Get("argscb").MustString(),
	}
	return event,nil
}
func ParseMeta(raw json.RawMessage)(interface{}, error){
	js, err := NewJson(raw)
	if err != nil {
		return nil, err
	}
	m :=js.Get("meta").MustMap()
	mk :=make(map[string]uint64)
	for k,v :=range m{
		value,_:=strconv.ParseUint(v.(string),10,64)
		mk[k]=value
	}
	//移除broker键值
	delete(mk,"broker")
	return &types.Meta{Meta:mk},nil
}
