package broker

import (
	"math/rand"
	"time"

	"gitlab.33.cn/link33/chain33-sdk-go/crypto"
	"gitlab.33.cn/link33/chain33-sdk-go/types"
)

// 构造发布跨链交易事件交易
func EmitInterchainEvent(paraName, dstServiceID, funcName string, args, argscb, argsrb string, txType uint64) *types.Transaction {
	payload := &types.BrokerAction{Ty: TyEmitInterchainEventAction, Value: &types.BrokerAction_EmitInterchainEvent{EmitInterchainEvent: &types.InterchainEvent{DstServiceID: dstServiceID, Type: txType, Func: funcName, Args: args, Argscb: argscb, Argsrb: argsrb}}}
	if paraName == "" {
		tx := &types.Transaction{Execer: []byte(BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: Addr}
		return tx
	} else {
		tx := &types.Transaction{Execer: []byte(paraName + BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: crypto.GetExecAddress(paraName + BrokerX)}
		return tx
	}
}

// 构造更新Index交易
func UpdateIndex(paraName, srcServiceID, dstServiceID string, sequenceNum, reqType uint64, status int32) *types.Transaction {
	payload := &types.BrokerAction{Ty: TyUpdateIndexAction, Value: &types.BrokerAction_UpdateIndex{UpdateIndex: &types.UpdateIndex{DstServiceID: dstServiceID, SrcServiceID: srcServiceID, ReqType: reqType, SequenceNum: sequenceNum, Response: &types.Response{Status:status}}}}
	if paraName == "" {
		tx := &types.Transaction{Execer: []byte(BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: Addr}
		return tx
	} else {
		tx := &types.Transaction{Execer: []byte(paraName + BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: crypto.GetExecAddress(paraName + BrokerX)}
		return tx
	}
}

func InitBroker(bxhId, appChainId string) *types.Transaction {
	payload := &types.BrokerAction{Ty: TyInitAction, Value: &types.BrokerAction_Init{Init: &types.Init{
		BxhId:      bxhId,
		AppChainId: appChainId,
	}}}
	tx := &types.Transaction{Execer: []byte(BrokerX), Payload: types.Encode(payload), Fee: 1e5, Nonce: rand.Int63n(time.Now().UnixNano()), To: Addr}
	return tx
}
